SUBROUTINE sub_poloffgrid_eg(index_h,index_y,a,h_out,c,s)
  ! Computes interpolated (unconditional) policy functions off nodes
  ! given consumption, saving, and value fns on endogenous grid nodes
  ! CONDITIONAL on housing choice
  ! Inputs:
  !         Current state 
  !	        index_h, index_y, a
  ! Outputs:
  !         Interpolated value for policy fns
  !         h_out, c, s
	
  USE mod_types
  USE mod_common_var
  USE mod_int_node
  USE mod_felicity

  IMPLICIT NONE

  INTEGER, INTENT(in)   :: index_h, index_y
  REAL(dp), INTENT(in)	:: a

  INTEGER, INTENT(out) 	:: h_out
  REAL(dp), INTENT(out)	:: c, s

  REAL(dp), DIMENSION(len_h_grid):: c_temp, s_temp, val, z_val
  REAL(dp)                       :: fel1, trans_cost
  REAL(dp), EXTERNAL  :: z
  INTEGER		      :: index_h_prime, k, node
  
  ! Value function conditional on housing choice index_h_prime
  DO index_h_prime = 1, len_h_grid
    z_val(index_h_prime) = z(a,index_h_prime,index_h,index_y) 

    ! Recover RIGHT interpolating nodes
    k = max_gr(index_h_prime,index_y)
    CALL sub_integm_node(zend_gr(1:k,index_h_prime,index_y),&
        z_val(index_h_prime),node)
    node = node - 1
    IF (node==0) THEN
      ! Borrowing constrained agents
      fel1 = fel(c_gr(1,index_h_prime,index_y))
      c_temp(index_h_prime) = z_val(index_h_prime) - a_grid(1)
      val(index_h_prime) = val_gr(1,index_h_prime,index_y) - fel1 + &
          fel(c_temp(index_h_prime))
    ELSE
      c_temp(index_h_prime) = c_gr(node,index_h_prime,index_y) + &
      ( c_gr(node+1,index_h_prime,index_y) - c_gr(node,index_h_prime,index_y) )/ &
      ( zend_gr(node+1,index_h_prime,index_y) - &
      zend_gr(node,index_h_prime,index_y) ) * &
      ( z_val(index_h_prime) - zend_gr(node,index_h_prime,index_y) )

    val(index_h_prime) = val_gr(node,index_h_prime,index_y) + &
        ( val_gr(node+1,index_h_prime,index_y) - &
        val_gr(node,index_h_prime,index_y) )/ &
        ( zend_gr(node+1,index_h_prime,index_y) - &
        zend_gr(node,index_h_prime,index_y) ) * &
        ( z_val(index_h_prime) - zend_gr(node,index_h_prime,index_y) )
    

    END IF

  END DO

  ! Current housing choice 
  h_out = MAXLOC(val,1)
  
  ! Current consumption and saving
  c = c_temp(h_out) 
  s = z_val(h_out) - c

  
END SUBROUTINE sub_poloffgrid_eg
