PROGRAM Main

   USE mod_types
   USE mod_common_var
  USE mod_interface
   USE mod_felicity

   IMPLICIT NONE

  REAL (dp), ALLOCATABLE :: yp_grid(:), pi_yp(:,:), picum_yp(:,:)
  REAL (dp), ALLOCATABLE :: yt_grid(:), pi_yt(:,:), picum_yt(:,:)

   REAL(dp)                             :: max_euler_err
   REAL (dp), DIMENSION(len_a_grid)		:: c_guess
   REAL(dp), DIMENSION(len_a_grid,len_h_grid,len_y_grid) :: val, opt_euler_err, diff

   REAL(dp), DIMENSION(len_a_grid,len_h_grid,len_h_grid,len_y_grid):: &
        con,sav,val_choice,euler_errors,exp_mu
   INTEGER, DIMENSION(len_a_grid,len_h_grid,len_y_grid) :: index_house
   INTEGER, DIMENSION(len_h_grid,len_y_grid) :: max_a_index
   INTEGER	:: h,i1,i2,j1,j2,index,index_prime,c(3)
   REAL :: t0,t1,t2,t3

   !Begins timing the main block.
  CALL CPU_TIME(t0)
   t1 = secnds(0.0)

   ! 0. Generate flag for felicity function
   !-------------------------------------------------------------

   IF (alpha.NE.1.d0 .AND. tau.NE.0.d0) THEN
     fel_flag = 1
   ELSEIF (alpha==1.d0) THEN
     IF (tau.NE.0.d0) THEN
       fel_flag = 2
     ELSE
       fel_flag = 3
     END IF
   ELSE
     fel_flag = 4
   END IF

   ! 1. Create grids
   !-------------------------------------------------------------
   !1.a Income grid
   !---------------------

   ALLOCATE (yp_grid(len_yp_grid),pi_yp(len_yp_grid,len_yp_grid),&
        picum_yp(len_yp_grid,len_yp_grid) )

   ALLOCATE (yt_grid(len_yt_grid),pi_yt(len_yt_grid,len_yt_grid),&
        picum_yt(len_yt_grid,len_yt_grid) )

  CALL sub_tauchen(yp_grid,pi_yp,picum_yp,rho_yp,mu0_yp,&
        sigma_yp,len_yp_grid,cover_yp)

  CALL sub_tauchen(yt_grid,pi_yt,picum_yt,rho_yt,mu0_yt,&
        sigma_yt,len_yt_grid,cover_yt)

   ! Income grid (vectorization of matrix yp_grid x yt_grid)
  DO i1 = 1,len_yp_grid
    DO j1 = 1,len_yt_grid
         index = len_yt_grid*(i1-1) + j1
         y_grid(index) = yp_grid(i1) + yt_grid(j1)
    END DO
  END DO

  y_grid = EXP(y_grid)

   ! Transition matrix for income
  DO i1 = 1, SIZE(pi_yp,1)
    DO i2 = 1, SIZE(pi_yt,1)
      DO j1 = 1, SIZE(pi_yp,2)
        DO j2 = 1, SIZE(pi_yt,2)
          pi_y((i1-1)*SIZE(pi_yt,1)+i2,(j1-1)*SIZE(pi_yt,2)+j2) = &
                    pi_yp(i1,j1)*pi_yt(i2,j2)
        END DO
      END DO
    END DO
  END DO

   ! Compute the CDF of the transition matrix.
   picum_y(:,1)=pi_y(:,1)
   DO i1 = 2,len_y_grid
      picum_y(:,i1)=picum_y(:,i1-1)+pi_y(:,i1)
   ENDDO

   ! 1.b Double exponential grid for assets over [min_a, max_a]
   !--------------------------
   CALL sub_grid(2,len_a_grid,min_a,max_a,a_grid)

   ! 1.c Uniform grid for housing over [min_h, max_h]
   !-------------------------
   CALL sub_grid(0,len_h_grid,min_h,max_h,h_grid)

   ! 2. Initial guesses for value fn
   !----------------------------------------

    DO i1 = 1,len_y_grid
       c_guess(:) = r*a_grid(:) + y_grid(i1)  
       DO i2 = 1,len_h_grid
         h_prime = h_grid(i2)
         val(:,i2,i1) = fel(c_guess)
       END DO
    END DO


   ! 3. Solving for value and policy fns
   !--------------------------------------------------------------
      IF (VFI_type==0) THEN
      ! Discretized VFI
      CALL sub_valueiter(index_house,con,sav,val_choice,val)
    ELSE
      ! VFI w/ linear interpolation
      CALL sub_valueiter2(index_house,con,sav,val_choice,val)
    END IF
    
    
    ! Elapsed time for the decision problem
    t2 = secnds(t1)
  	CALL CPU_TIME(t3)

    PRINT *, ' '
    PRINT *, 'Elapsed cpu time in the main program =', t3-t0, 'secs'
    PRINT *, ' '

    PRINT *, ' '
    PRINT *, 'Elapsed user time in the main program =', t2, 'secs'
    PRINT *, ' '

  OPEN(1,FILE='Output.txt',POSITION='REWIND',ACTION='WRITE')
  WRITE (1,90) alpha, tau, bita, r, max_a, max_h, len_a_grid, toler
90 FORMAT (' alpha=', F7.4, ' tau=', F7.4, ' beta=', F3.2, ' r=', &
       F3.2, ' max_a=', F4.0, ' max_h=', F4.0, ' len_a_grid=', I4, &
       ' toler=', E10.4)
  WRITE (1,*) '====================================================================='
    WRITE (1,*) ' '
    WRITE (1,*) 'Elapsed cpu time in the main program =', t3-t0, 'secs'
    WRITE (1,*) ' '
    WRITE (1,*) ' '
    WRITE (1,*) 'Elapsed user time in the main program =', t2, 'secs'
    WRITE (1,*) ' '
    CLOSE(1)

  ! 4. Checking whether upper bounds on grids are binding
   !--------------------------------------------------------------
     
    DO i1 = 1,len_y_grid
      DO i2 = 1,len_h_grid 
        DO j1= 1,len_a_grid
          diff(j1,i2,i1) = sav(j1,index_house(j1,i2,i1),i2,i1)-a_grid(j1)
        END DO
      END DO
    END DO
        
    IF (ANY(diff(len_a_grid,:,:)>0.d0)) THEN
      ! Upper bound on wealth grid is binding
      PRINT *, "Too narrow asset range"
    STOP
    ELSE
      ! Check upper bound on housing grid

    ! Asset level for which saving function crosses 45 line 
    ! as a fn of income and housing  
    max_a_index = MAXLOC(diff,1,diff.LE.0.d0)

     
    ! Find the maximum point on the durable grid before which 
    ! an individual with highest income and wealth downsizes
      index = index_house(1,1,len_y_grid)
      DO i1 = 1,100   
        index_prime = index_house(max_a_index(index,len_y_grid),&
            index,len_y_grid)
        IF (index_prime>index) THEN
          index = index_prime
        ELSE 
          PRINT *, "Max wealth index", max_a_index(index,len_y_grid), &
              "Max dur. index", index
        OPEN(1,FILE='Output.txt',POSITION='APPEND',ACTION='WRITE') 
          WRITE (1,*) "Max wealth index", max_a_index(index,len_y_grid), &
              "Max dur. index", index
          CLOSE(1)
          EXIT
        ENDIF
      END DO
    END IF
    

  ! 5. Calculates max and average Euler error along a simulated path.
    !-------------------------------------------------------------------
    CALL sub_simuldyneuler(con,sav,val_choice,picum_y) 
    DEALLOCATE (yp_grid,pi_yp,picum_yp)
    DEALLOCATE (yt_grid,pi_yt,picum_yt)

  ! 6. Saves  policy fns to file for all asset grid points
    !-------------------------------------------------------------------
  h = 1 ! Initial housing stock
  c =(/1, 1+INT(len_y_grid/2), len_y_grid /) ! Three income levels

  OPEN(1,FILE='Policy_fns.csv',POSITION='REWIND',ACTION='WRITE')
  DO j1= 1,len_a_grid
    WRITE(1,'(F6.3, 1X, F6.3, 1X, F6.3, 1X, F6.3, 1X, &
        & F6.3, 1X, F6.3, 1X, F6.3, 1X, &
        & I2, 1X, I2, 1X, I2)') &
        & a_grid(j1), (con(j1,index_house(j1,h,c(i1)),1,c(i1)),i1=1,3), &
        & (sav(j1,index_house(j1,h,c(i1)),h,c(i1)),i1=1,3), &
        & (index_house(j1,h,c(i1)),i1=1,3)
  END DO
  CLOSE(1)

 END PROGRAM main
