MODULE mod_fcn
! Contains various functions used by the non-linear solver sub_newton.
  USE mod_types
  USE mod_common_var
  USE mod_felicity
  IMPLICIT NONE
CONTAINS

  SUBROUTINE sub_fcn(N,X,F)
    !Returns felicity of consumption

    INTEGER, INTENT(in)   	:: N
    REAL(dp), INTENT(in)	:: X(N)
    REAL(dp), INTENT(out)	:: F(N)  

    F = fel(X)

  END SUBROUTINE sub_fcn

  SUBROUTINE sub_dfcn(N,X,DF)
    !Returns marginal felicity of consumption

    INTEGER, INTENT(in)   :: N
    REAL(dp), INTENT(in)  :: X(N)
    REAL(dp), INTENT(out) :: DF(N)  

    DF = dfel(X)

  END SUBROUTINE sub_dfcn

  SUBROUTINE sub_d2fcn(N,X,D2F)
    !Returns derivative of marginal utility of consumption

    INTEGER, INTENT(in)   :: N
    REAL(dp), INTENT(in)  :: X(N)
    REAL(dp), INTENT(out) :: D2F(N)  

    D2F = d2fel(X)

  END SUBROUTINE sub_d2fcn

  SUBROUTINE sub_fcn1(N,X,F)
    !Returns Euler equation. b_global is expected future utility.

    INTEGER, INTENT(in)   	:: N
    REAL(dp), INTENT(in)	:: X(N)
    REAL(dp), INTENT(out)	:: F(N)  
  
    F = dfel(X) - b_global

  END SUBROUTINE sub_fcn1

  SUBROUTINE sub_min_z(N,Z,F)

    ! Called by a non-linear solver to find the lowest value of cash at
    ! hand z_min below which the optimum saving switches to the borrowing
    ! constraint a_grid(1).   
    ! If the lowest optimal value of future assets aprime_node(1) found
    ! by the egm is such that aprime_node(1) > a_grid(1), z_min is the zero of
    ! u(z-a_grid(1)) + exp_val(agrid(1),.) = u(z-aprime_node(1)+ exp_val(aprime_node(1),.)
    !
    ! The global variables b_global and fut_v1_global are assigned the
    ! following values before the procedure is called:
    ! b_global = aprime_node(1) 
    ! fut_v1_global = exp_val(aprime_node(1),.) - exp_val(agrid(1),.) 
    INTEGER, INTENT(in)   	:: N
    REAL(dp), INTENT(in)	:: Z(N)
    REAL(dp), INTENT(out)	:: F(N)  

    F = fel(Z - a_grid(1)) - fel(Z - b_global) - fut_v1_global

  END SUBROUTINE sub_min_z

  SUBROUTINE sub_dmin_z(N,Z,DF)
    ! Derivative of the function in sub_min_z

    INTEGER, INTENT(in)   	:: N
    REAL(dp), INTENT(in)	:: Z(N)
    REAL(dp), INTENT(out)	:: DF(N)  

    DF = dfel(Z - a_grid(1)) - dfel(Z - b_global)

  END SUBROUTINE sub_dmin_z

END MODULE mod_fcn
