SUBROUTINE sub_exp_mu(con,sav,index_house,val_choice,exp_mu)
  ! Calculates future marginal utility of consumption
  ! given consumption, saving, housing  and value fns
  ! Inputs:
  !	con, sav and  val_choice: consumption, saving and value fns
  !	    CONDITIONAL on housing choice
  ! 	index_house:              policy fn for housing
  ! Outputs:
  ! 	exp_mu: undiscounted expected marginal utility 
  !	    CONDITIONAL on current housing choice.
	
  USE mod_types
  USE mod_common_var
  USE mod_int_node
  USE mod_felicity

  IMPLICIT NONE

  REAL(dp), INTENT(in), DIMENSION(:,:,:,:)	:: con, sav, val_choice
  INTEGER, INTENT(in), DIMENSION(:,:,:)		:: index_house
  REAL(dp), INTENT(out), &
       DIMENSION(SIZE(con,1),SIZE(con,2),SIZE(con,3),SIZE(con,4)) :: exp_mu

  REAL(dp)				:: a_prime, fut_con
  REAL(dp), DIMENSION(len_h_grid)	:: val
  REAL(dp), DIMENSION(len_y_grid)	:: fut_mu
  INTEGER			 	:: fut_house, node
  INTEGER	:: i, index_a, index_h, index_h_prime, index_y, index_y_prime
  
  exp_mu = 0.d0

  DO i = 1, len_h_grid ! I Housing choice
    DO index_h = 1, len_h_grid ! II Initial housing stock
      DO index_y = 1, len_y_grid ! III 
        DO index_a = 1, len_a_grid ! IV
          ! Compute exp_mu 
          
          ! Saving  given current state (index_a,index_h,index_y )
          a_prime = sav(index_a,i,index_h,index_y )
          
          ! Calculating fut. marg. util. given today's choice(i,a_prime) 
          !---------------------------------------
          DO index_y_prime = 1, len_y_grid
            DO index_h_prime = 1, len_h_grid
              ! Recover RIGHT interpolating nodes
              CALL sub_int_node(a_grid,a_prime,node)
              node = node - 1
              val(index_h_prime) = &
                  val_choice(node,index_h_prime,i,index_y_prime) + &
                  ( val_choice(node+1,index_h_prime,i,index_y_prime)- &
                  val_choice(node,index_h_prime,i,index_y_prime) ) / &
                  ( a_grid(node+1) - a_grid(node) ) * &
                  ( a_prime - a_grid(node) )
            END DO
              
            !Next period housing choice in 
            !state (a_prime,i,index_y_prime)
            fut_house = MAXLOC(val,1)
              
            !Next period consumption 
            fut_con = &
                con(node,fut_house,i,index_y_prime) + &
                ( con(node+1,fut_house,i,index_y_prime)- &
                con(node,fut_house,i,index_y_prime) ) / &
                ( a_grid(node+1) - a_grid(node) ) * &
                ( a_prime - a_grid(node) )
              
            !Next period marginal utility 
            h_prime = h_grid(fut_house)
            fut_mu(index_y_prime) = dfel(fut_con)
            
          END DO
                      
          ! Expected fut. marginal utility given present state
          ! (index_a,index_h,index_y) and housing choice i
          exp_mu(index_a,i,index_h,index_y) = &
              &DOT_PRODUCT(pi_y(index_y,:),fut_mu(:)) 
          
        END DO ! IV
      END DO ! III
    END DO ! II
  END DO ! I
  
END SUBROUTINE sub_exp_mu
