SUBROUTINE sub_poloffgrid(index_h,index_y,a,h_out,c,s)
  ! Computes interpolated (unconditional) policy functions off nodes
  ! given consumption, saving, and value fns on nodes
  ! CONDITIONAL on housing choice
  ! Inputs:
  !         Current state 
  !	        index_h, index_y, a
  ! Outputs:
  !         Interpolated value for policy fns
  !         h_out, c, s
	
  USE mod_types
  USE mod_common_var
  USE mod_int_node
  USE mod_felicity

  IMPLICIT NONE

  INTEGER, INTENT(in)   :: index_h, index_y
  REAL(dp), INTENT(in)	:: a

  INTEGER, INTENT(out) 	:: h_out
  REAL(dp), INTENT(out)	:: c, s

  REAL(dp), DIMENSION(len_h_grid):: c_temp, s_temp, val, z_val
  REAL(dp)                       :: fel1, trans_cost
  REAL(dp), EXTERNAL  :: z
  INTEGER		         :: index_h_prime, k, node
  INTEGER, DIMENSION(len_h_grid) :: nodes
  
  ! Value function conditional on housing choice index_h_prime
  DO index_h_prime = 1, len_h_grid
    z_val(index_h_prime) = z(a,index_h_prime,index_h,index_y) 

    ! Recover RIGHT interpolating nodes
    CALL sub_int_node(a_grid,&
        z_val(index_h_prime),node)
    node = node - 1
    nodes(index_h_prime) = node

    val(index_h_prime) = val_grid(node,index_h_prime,index_y) + &
        ( val_grid(node+1,index_h_prime,index_y) - &
        val_grid(node,index_h_prime,index_y) )/ &
        ( a_grid(node+1) - &
        a_grid(node) ) * &
        ( z_val(index_h_prime) - a_grid(node) )

  END DO

  ! Current housing choice 
  h_out = MAXLOC(val,1)
  
  ! Current consumption and saving
  c = c_grid(nodes(h_out),h_out,index_y) + &
      ( c_grid(nodes(h_out)+1,h_out,index_y) - &
      c_grid(nodes(h_out),h_out,index_y) )/ &
      ( a_grid(nodes(h_out)+1) - a_grid(nodes(h_out)) ) * &
      ( z_val(h_out) - a_grid(nodes(h_out)) )
  s = z_val(h_out) - c

  
END SUBROUTINE sub_poloffgrid
